@@ -1,6 +1,9 @@
+import math
 import torch
+import torch.nn as nn
 import numpy as np
 import scipy.signal
+from typing import Optional
 from flamo.utils import RegularGridInterpolator
 
 
@@ -69,6 +72,68 @@ def get_frequency_samples(num: int, device: str | torch.device = None):
     return torch.polar(abs, angle * np.pi)
 
 
+class HadamardMatrix(nn.Module):
+    """
+    Generate a Hadamard matrix of size N as a nn.Module.
+    """
+
+    def __init__(self, N, device: Optional[str] = None):
+        super().__init__()
+        self.N = N
+        self.device = device
+
+    def forward(self, x):
+        U = torch.tensor([[1.0]], device=self.device)
+        while U.shape[0] < self.N:
+            U = torch.kron(
+                U, torch.tensor([[1, 1], [1, -1]], dtype=U.dtype, device=U.device)
+            ) / torch.sqrt(torch.tensor(2.0, device=U.device))
+        return U
+
+
+class RotationMatrix(nn.Module):
+    """
+    Generate a rotation matrix of size N as a nn.Module from a given angle.
+    """
+
+    def __init__(
+        self,
+        N: int,
+        min_angle: float = 0,
+        max_angle: float = torch.pi / 4,
+        iter: Optional[int] = None,
+        device: Optional[str] = None,
+    ):
+
+        super().__init__()
+        self.N = N
+        self.min_angle = min_angle
+        self.max_angle = max_angle
+        self.iter = iter
+        self.device = device
+
+    def create_submatrix(self, angles: torch.Tensor, iters: int = 1):
+        """Create a submatrix for each group."""
+        X = torch.zeros(2, 2, device=self.device)
+        angles[0] = torch.clamp(angles[0], self.min_angle, self.max_angle)
+        X.fill_diagonal_(torch.cos(angles[0]))
+        X[1, 0] = -torch.sin(torch.tensor(angles[0], device=self.device))
+        X[0, 1] = torch.sin(torch.tensor(angles[0], device=self.device))
+
+        if iters is None:
+            iters = torch.log2(torch.tensor(self.N)).int().item() - 1
+        for i in range(iters):
+            if len(angles) > 1:
+                X = torch.kron(X, self.create_submatrix([angles[i]]))
+            else:
+                X = torch.kron(X, X)
+        return X
+
+    def forward(self, theta):
+
+        return self.create_submatrix(theta, self.iter)
+
+
 def biquad2tf(b: torch.Tensor, a: torch.Tensor, nfft: int):
     r"""
     Converts a biquad filter representation to a transfer function.
@@ -111,11 +176,12 @@ def signal_gallery(
         - ``sweep``: A linear sweep from 20 Hz to 20 kHz.
         - ``wgn``: White Gaussian noise.
         - ``exp``: An exponential decay signal.
+        - ``velvet``: A velvet noise signal with density :attr:`rate` impulses per second.
         - ``reference``: A reference signal provided as argument :attr:`reference`.
 
         **Arguments**:
             - **batch_size** (int): The number of batches to generate.
-            - **n_samples** (int): THe signal length in samples.
+            - **n_samples** (int): The signal length in samples.
             - **n_channel** (int): The number of channels in each signal.
             - **signal_type** (str, optional): The type of signal to generate. Defaults to 'impulse'.
             - **fs** (int, optional): The sampling frequency of the signals. Defaults to 48000.
@@ -133,10 +199,12 @@ def signal_gallery(
         "wgn",
         "exp",
         "reference",
+        "noise",
+        "velvet",
     }
 
     if signal_type not in signal_types:
-        raise ValueError(f"Matrix type {signal_type} not recognized.")
+        raise ValueError(f"Signal type {signal_type} not recognized.")
     match signal_type:
         case "impulse":
             x = torch.zeros(batch_size, n_samples, n)
@@ -178,6 +246,12 @@ def signal_gallery(
                 .expand(batch_size, n_samples, n)
                 .to(device)
             )
+        case "velvet":
+            x = torch.empty((batch_size, n_samples, n), device=device)
+            for i_batch in range(batch_size):
+                for i_ch in range(n):
+                    x[i_batch, :, i_ch] = gen_velvet_noise(n_samples, fs, rate, device)
+            return x
         case "reference":
             if isinstance(reference, torch.Tensor):
                 return reference.expand(batch_size, n_samples, n).to(device)
@@ -185,8 +259,42 @@ def signal_gallery(
                 return torch.tensor(reference, device=device).expand(
                     batch_size, n_samples, n
                 )
+        case "noise":
+            return torch.randn((batch_size, n_samples, n), device=device)
 
 
+def gen_velvet_noise(n_samples: int, fs: int, density: float, device: str | torch.device = None) -> torch.Tensor:
+    r"""
+    Generate a velvet noise sequence.
+    **Arguments**:
+        - **n_samples** (int): The length of the signal in samples.
+        - **fs** (int): The sampling frequency of the signal in Hz.
+        - **density** (float): The density of impulses in impulses per second.
+        - **device** (str | torch.device): The device of constructed tensors.
+    **Returns**:
+        - torch.Tensor: A tensor of shape (n_samples,) containing the velvet noise sequence.
+    """
+    Td = fs / density # average distance between impulses
+    num_impulses = n_samples / Td # expected number of impulses
+    floor_impulses = math.floor(num_impulses)
+    grid = torch.arange(floor_impulses) * Td
+
+    jitter_factors = torch.rand(floor_impulses)
+    impulse_indices = torch.ceil(grid + jitter_factors * (Td - 1)).long()
+
+    # first impulse is at position 0 and all indices are within bounds
+    impulse_indices[0] = 0
+    impulse_indices = torch.clamp(impulse_indices, max=n_samples - 1)
+            
+    # Generate random signs (+1 or -1)
+    signs = 2 * torch.randint(0, 2, (floor_impulses,)) - 1
+            
+    # Construct sparse signal
+    sequence = torch.zeros(n_samples, device=device)
+    sequence[impulse_indices] = signs.float()
+
+    return sequence
+
 def hertz2rad(hertz: torch.Tensor, fs: int):
     r"""
     Convert frequency from Hz to rad.
